require File.expand_path(File.join(File.dirname(__FILE__), 'spec_helper'))

RSpec.describe Appforce::Spawn::Api::Call do
  before do
    logger = Logger.new('/dev/null')
    Appforce::Spawn.logger = logger
    Appforce::Config.load_config(file_fixture_path('appforce_config.yml'))
  end

  describe '.latest_version' do
    before do
      @last_modified = Date.new(2010, 1, 15).to_s
      @content_length = '1024'
      @request_object = HTTParty::Request.new Net::HTTP::Get, '/'
      @response_object = Net::HTTPOK.new('1.1', 200, 'OK')
      allow(@response_object).to receive_messages(body: '{"version":"0.1"}')
      @response_object['last-modified'] = @last_modified
      @response_object['content-length'] = @content_length
      @parsed_response = lambda { {"version" => "0.1"} }
      @response = HTTParty::Response.new(@request_object, @response_object, @parsed_response)
      allow(HTTParty).to receive(:get).and_return(@response)
    end
    
    it 'should return the latest version number from RubyGems' do
      info = Appforce::Spawn::Api::Call.latest_version
      expect(info).to be_a String
      expect(info).to eq '0.1'
    end
  end
  
  describe '.get_hosts' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_hosts(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_hosts(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.get_vars' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_vars(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_vars(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.get_active_users' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_active_users(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_active_users(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.get_inactive_users' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_inactive_users(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_inactive_users(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end
  
  describe '.get_private_key' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_private_key(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_private_key(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.ping' do
    context 'Host is available' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return 200' do
        ping = Appforce::Spawn::Api::Call.ping
        expect(ping.code).to eq 200
        expect(ping.value).to eq 'good'
      end
    end
  end

  describe '.get_clients' do
    context 'Host is available' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return 200' do
        ping = Appforce::Spawn::Api::Call.get_clients
        expect(ping.code).to eq 200
        expect(ping.value).to eq 'good'
      end
    end
  end

  describe '.get_all_host_data' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_all_host_data(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_all_host_data(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.get_main_scout' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_main_scout(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_main_scout(*[nil, {:client_api_name => 'template_test'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.get_host_scout_vars' do
    context 'missing parameters (Host API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Api::Call.get_host_scout_vars(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'has Client API Name parameter' do
      before do
        resp = double('resp')
        allow(resp).to receive(:code).and_return(200)
        allow(resp).to receive(:value).and_return('good')
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return an API response' do
        api_resp = Appforce::Spawn::Api::Call.get_host_scout_vars(*[nil, {:host_api_name => '1'}])
        expect(api_resp.value).to eq 'good'
      end
    end
  end

  describe '.list_clients' do
    context 'Host is available' do
      before do
        resp = Enumerator.new do |y|
          1.times do |x|
            y << {"name"=>"test", "api_name"=>"test"}
          end
        end
        allow(resp).to receive(:value).and_return('good')
        allow(resp).to receive(:code).and_return(200)
        allow(HTTParty).to receive(:get).and_return(resp)
      end

      it 'should return 200' do
        expect { Appforce::Spawn::Api::Call.list_clients }.to output(" == Client List ==============================================================\n Client: test             API Name: test\n =============================================================================\n").to_stdout
      end
    end
  end
  
  describe '.list_hosts' do
    context 'Host is available' do
      before do
        allow(Appforce::Spawn::Api::Call).to receive(:get_all_host_data).and_return(file_fixture_to_json('all_host_data.json'))
      end

      it 'should write a list of hosts to screen + return the JSON list of hosts' do
        resp = nil
        expect { resp = Appforce::Spawn::Api::Call.list_hosts }.to output(" == Host List ================================================================\n 1 - 'ubuntu-test01' host: ubuntu-test01 ip: 192.168.10.3\n =============================================================================\n").to_stdout
        expect(resp).to eq file_fixture_to_json('all_host_data.json')
      end
    end
  end

  describe '.ssh_to_host' do
    context 'Host is available' do
      before do
        allow(Appforce::Spawn::Api::Call).to receive(:get_all_host_data).and_return(file_fixture_to_json('all_host_data.json'))
        allow(Appforce::Spawn::Api::Call).to receive(:ssh_cmd).and_return('touch tmp/fork')
        allow(HighLine).to receive(:ask).and_return(1)
      end

      it 'should execute the command returned by the ssh_cmd call based on the input from the user' do
        expect { Appforce::Spawn::Api::Call.ssh_to_host }.to output(" == Host List ================================================================\n 1 - 'ubuntu-test01' host: ubuntu-test01 ip: 192.168.10.3\n =============================================================================\n").to_stdout
      end
    end
  end

  context 'private methods' do
    describe '.ssh_cmd' do
      it 'should return a valid ssh command' do
        params = {'hostname' => 'test', 'username' => 'user', 'method' => 'hostname' }
        expect(Appforce::Spawn::Api::Call.send(:ssh_cmd, params)).to eq 'ssh user@test'
      end
    end

    describe '.base_api_url' do
      it 'should return the API URL driven by the .appforce config' do
        expect(Appforce::Spawn::Api::Call.send(:base_api_url)).to eq 'http://afuka.synctree.com/VERSION_STRING/'
      end
    end

    describe '.client_url' do
      it 'should return the API URL for the Client' do
        expect(Appforce::Spawn::Api::Call.send(:client_url, 'test')).to eq 'http://afuka.synctree.com/VERSION_STRING/client/test'
      end
    end

    describe '.token_param' do
      it 'should return the token parameter string' do
        expect(Appforce::Spawn::Api::Call.send(:token_param)).to eq 'token=INVALID_TOKEN'
      end
    end

    describe '.make_api_call' do
      context 'get a valid API response' do
        before do
          resp = double('resp')
          allow(resp).to receive(:code).and_return(200)
          allow(resp).to receive(:value).and_return('good')
          allow(HTTParty).to receive(:get).and_return(resp)
        end

        it 'should return the response object' do
          api_resp = Appforce::Spawn::Api::Call.send(:make_api_call, 'url')
          expect(api_resp.value).to eq 'good'
        end
      end

      context 'use an invalid token' do
        before do
          resp = double('resp')
          allow(resp).to receive(:code).and_return(401)
          allow(resp).to receive(:value).and_return('invalid')
          allow(HTTParty).to receive(:get).and_return(resp)
        end

        it 'should raise an Appforce::Spawn::API::UnauthorizedAccess exception' do
          expect { Appforce::Spawn::Api::Call.send(:make_api_call, 'url') }.to raise_exception(Appforce::Spawn::API::UnauthorizedAccess)
        end
      end

      context 'hit an invalid route' do
        before do
          resp = double('resp')
          allow(resp).to receive(:code).and_return(404)
          allow(resp).to receive(:value).and_return('invalid')
          allow(HTTParty).to receive(:get).and_return(resp)
        end

        it 'should raise an Appforce::Spawn::API::NotFound exception' do
          expect { Appforce::Spawn::Api::Call.send(:make_api_call, 'url') }.to raise_exception(Appforce::Spawn::API::NotFound)
        end
      end

      context 'service is unavailable' do
        before do
          resp = double('resp')
          allow(resp).to receive(:code).and_return(502)
          allow(resp).to receive(:value).and_return('invalid')
          allow(HTTParty).to receive(:get).and_return(resp)
        end

        it 'should raise an Appforce::Spawn::API::Unavailable exception' do
          expect { Appforce::Spawn::Api::Call.send(:make_api_call, 'url') }.to raise_exception(Appforce::Spawn::API::Unavailable)
        end
      end

      context 'HTTParty raises an exception' do
        before do
          allow(HTTParty).to receive(:get).and_raise('BOOM')
        end

        it 'should raise an Appforce::Spawn::API::CallFailure exception' do
          expect { Appforce::Spawn::Api::Call.send(:make_api_call, 'url') }.to raise_exception(Appforce::Spawn::API::CallFailure)
        end
      end
    end
  end

end
