require File.expand_path(File.join(File.dirname(__FILE__), 'spec_helper'))

RSpec.describe Appforce::Spawn::Api::Call do
  before do
    logger = Logger.new('/dev/null')
    Appforce::Spawn.logger = logger
    Appforce::Config.load_config(file_fixture_path('appforce_config.yml'))
  end

  describe '.can_run?' do
    context 'all files exists' do
      before do
        allow(File).to receive(:exist?).with('./vars.yml') { true }
        allow(File).to receive(:exist?).with('./active_users.yml') { true }
        allow(File).to receive(:exist?).with('./inactive_users.yml') { true }
        allow(File).to receive(:exist?).with('./hosts') { true }
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'should return true' do
        expect(Appforce::Spawn::Runner.can_run?).to be_truthy
      end
    end

    context 'vars.yml file is missing' do
      before do
        allow(File).to receive(:exist?).with('./vars.yml') { false }
        allow(File).to receive(:exist?).with('./active_users.yml') { true }
        allow(File).to receive(:exist?).with('./inactive_users.yml') { true }
        allow(File).to receive(:exist?).with('./hosts') { true }
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'should raise a Appforce::Spawn::MissingFile exception' do
        expect { Appforce::Spawn::Runner.can_run? }.to raise_exception(Appforce::Spawn::MissingFile)
      end
    end

    context 'active_users.yml file is missing' do
      before do
        allow(File).to receive(:exist?).with('./vars.yml') { true }
        allow(File).to receive(:exist?).with('./active_users.yml') { false }
        allow(File).to receive(:exist?).with('./inactive_users.yml') { true }
        allow(File).to receive(:exist?).with('./hosts') { true }
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'should raise a Appforce::Spawn::MissingFile exception' do
        expect { Appforce::Spawn::Runner.can_run? }.to raise_exception(Appforce::Spawn::MissingFile)
      end
    end

    context 'inactive_users.yml file is missing' do
      before do
        allow(File).to receive(:exist?).with('./vars.yml') { true }
        allow(File).to receive(:exist?).with('./active_users.yml') { true }
        allow(File).to receive(:exist?).with('./inactive_users.yml') { false }
        allow(File).to receive(:exist?).with('./hosts') { true }
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'should raise a Appforce::Spawn::MissingFile exception' do
        expect { Appforce::Spawn::Runner.can_run? }.to raise_exception(Appforce::Spawn::MissingFile)
      end
    end
    
    context 'hosts file is missing' do
      before do
        allow(File).to receive(:exist?).with('./vars.yml') { true }
        allow(File).to receive(:exist?).with('./active_users.yml') { true }
        allow(File).to receive(:exist?).with('./inactive_users.yml') { true }
        allow(File).to receive(:exist?).with('./hosts') { false }
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'should raise a Appforce::Spawn::MissingFile exception' do
        expect { Appforce::Spawn::Runner.can_run? }.to raise_exception(Appforce::Spawn::MissingFile)
      end
    end
  end

  context 'all files are available and runner can run' do
    before do
      allow(Appforce::Spawn::Runner).to receive(:can_run?).and_return(true)
      allow(File).to receive(:open).and_return(file_fixture('vars.yml'))
      allow(Appforce::Spawn::Runner).to receive(:system).and_return(true)
    end

    describe '.run_playbook' do
      it 'will log out info and execute a system command' do
        Appforce::Spawn::Runner.run_playbook
      end
    end

    describe '.run_dryrun' do
      it 'will log out info and execute a system command' do
        Appforce::Spawn::Runner.run_dryrun
      end
    end

    describe '.ansible_ping' do
      it 'will log out info and execute a system command' do
        Appforce::Spawn::Runner.ansible_ping
      end
    end

    describe '.display_ansible_command' do
      it 'will write the ansible command to the logger' do
        Appforce::Spawn::Runner.display_ansible_command
      end
    end

    describe '.display_ansible_ping_command' do
      it 'will write the ansible ping command to the logger' do
        Appforce::Spawn::Runner.display_ansible_ping_command
      end
    end
  end

  describe '.check_dependencies' do
    context 'ansible not installed' do
      before do
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(nil)
      end

      it 'will raise an exception' do
        expect { Appforce::Spawn::Runner.send(:check_dependencies) }.to raise_exception
      end
    end

    context 'ansible version is bad' do
      before do
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(nil)
      end

      it 'will raise an exception' do
        expect { Appforce::Spawn::Runner.send(:check_dependencies) }.to raise_exception
      end
    end

    context 'ansible galaxy role for rvm is not installed' do
      before do
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(nil)
      end

      it 'will raise an exception' do
        expect { Appforce::Spawn::Runner.send(:check_dependencies) }.to raise_exception
      end
    end

    context 'all dependacies are met' do
      before do
        allow(Appforce::Spawn::Runner).to receive(:ansible_installed?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:good_ansible_version?).and_return(true)
        allow(Appforce::Spawn::Runner).to receive(:rvm_galaxy_installed?).and_return(true)
      end

      it 'will return true' do
        expect(Appforce::Spawn::Runner.send(:check_dependencies)).to be_truthy
      end
    end
  end

  context 'private methods' do
    describe '.ansible_installed?' do
      context 'returns the result of a match data (true)' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('which ansible').and_return('/usr/bin/ansible')
        end

        it 'return a non nil value' do
          expect(Appforce::Spawn::Runner.send(:ansible_installed?)).to_not be_nil
        end
      end

      context 'returns the result of a match data (false)' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('which ansible').and_return('')
        end

        it 'it will return nil' do
          expect(Appforce::Spawn::Runner.send(:ansible_installed?)).to be_nil
        end
      end
    end

    describe '.good_ansible_version?' do
      context 'version is good' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('ansible --version').and_return("ansible 1.8.4\n  configured module search path = None\n")
        end

        it 'return a non nil value' do
          expect(Appforce::Spawn::Runner.send(:good_ansible_version?)).to_not be_nil
        end
      end

      context 'version is bad' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('ansible --version').and_return("ansible 1.8.2\n  configured module search path = None\n")
        end

        it 'it will return nil' do
          expect(Appforce::Spawn::Runner.send(:good_ansible_version?)).to be_nil
        end
      end
    end

    describe '.rvm_galaxy_installed?' do
      context 'role is installed' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('ansible-galaxy list').and_return("- rvm_io.rvm1-ruby, v1.3.5\n")
        end

        it 'return a non nil value' do
          expect(Appforce::Spawn::Runner.send(:rvm_galaxy_installed?)).to_not be_nil
        end
      end

      context 'role is not installed' do
        before do
          allow(Appforce::Spawn::Runner).to receive(:`).with('ansible-galaxy list').and_return("")
        end

        it 'it will return nil' do
          expect(Appforce::Spawn::Runner.send(:rvm_galaxy_installed?)).to be_nil
        end
      end
    end

    describe '.ansible_command' do
      context 'it fails to load the vars.yml file' do
        before do
          allow(File).to receive(:open).and_raise(Exception)
        end

        it 'it will raise an Excpetion' do
          expect { Appforce::Spawn::Runner.send(:ansible_command) }.to raise_exception
        end
      end

      context 'it successfully loads vars.yml file' do
        before do
          allow(File).to receive(:open).and_return(file_fixture('vars.yml'))
        end

        it 'will return the ansible command with vars' do
          expect(Appforce::Spawn::Runner.send(:ansible_command)).to match /ansible_user=username/
        end
      end

      context 'it successfully loads vars.yml file with a private key' do
        before do
          allow(File).to receive(:open).and_return(file_fixture('private_key_vars.yml'))
        end

        it 'will return the ansible command with vars' do
          expect(Appforce::Spawn::Runner.send(:ansible_command)).to match /private-key=/
        end
      end
    end

    describe '.ansible_ping_command' do
      context 'it fails to load the vars.yml file' do
        before do
          allow(File).to receive(:open).and_raise(Exception)
        end

        it 'it will raise an Excpetion' do
          expect { Appforce::Spawn::Runner.send(:ansible_ping_command) }.to raise_exception
        end
      end

      context 'it successfully loads vars.yml file' do
        before do
          allow(File).to receive(:open).and_return(file_fixture('vars.yml'))
        end

        it 'will return the ansible command with vars' do
          expect(Appforce::Spawn::Runner.send(:ansible_ping_command)).to match /username/
        end
      end

      context 'it successfully loads vars.yml file with a private key' do
        before do
          allow(File).to receive(:open).and_return(file_fixture('private_key_vars.yml'))
        end

        it 'will return the ansible command with vars' do
          expect(Appforce::Spawn::Runner.send(:ansible_ping_command)).to match /private-key=/
        end
      end
    end
  end
end
