require File.expand_path(File.join(File.dirname(__FILE__), 'spec_helper'))

RSpec.describe Appforce::Spawn::Template do
  before do
    logger = Logger.new('/dev/null')
    Appforce::Spawn.logger = logger
    Appforce::Config.load_config(file_fixture_path('appforce_config.yml'))
  end

  describe '.run_path' do
    it 'should return the current run path' do
      expect(Appforce::Spawn::Template.run_path).to eq Dir.pwd
    end
  end

  describe '.gem_path' do
    it 'should return the path of the directory the gem is run from' do
      path = Appforce::Spawn::Template.gem_path
      expect(path).to end_with "lib/appforce/spawn"
      expect(File.exist?("#{path}/version.rb")).to be_truthy
    end
  end

  describe '.ansible_source_path' do
    it 'should return the path of the directory for the Ansible source' do
      path = Appforce::Spawn::Template.ansible_source_path
      expect(File.exist?("#{path}/site.yml")).to be_truthy
    end
  end

  describe '.ansible_dest_path' do
    it 'should return a path to the client api name in the root directory' do
      path = Appforce::Spawn::Template.ansible_dest_path('testclient')
      expect(path).to end_with "testclient"
      expect(path).to match Dir.pwd
    end
  end

  describe '.create_dirs' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Template.create_dirs(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'it has correct parameters' do
      before do
        allow(Appforce::Spawn::Template).to receive(:ansible_dest_path).and_return("tmp/template_test")
      end

      it 'should create directories for the template' do
        Appforce::Spawn::Template.create_dirs(*[nil, {:client_api_name => 'template_test'}])
        expect(Dir.exist?("tmp/template_test")).to be_truthy
      end
    end

    context 'it has correct parameters & dir already exists' do
      before do
        allow(Appforce::Spawn::Template).to receive(:ansible_dest_path).and_return("tmp/template_test")
        FileUtils.mkdir("tmp/template_test") rescue true
      end

      it 'should handle the Errno::EEXIST exception and move on' do
        Appforce::Spawn::Template.create_dirs(*[nil, {:client_api_name => 'template_test'}])
        expect(Dir.exist?("tmp/template_test")).to be_truthy
      end

      after do
        `rm -rf tmp/template_test`
      end
    end
  end
  
  describe '.copy_template' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Template.copy_template(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'it has correct parameters' do
      before do
        allow(Appforce::Spawn::Template).to receive(:ansible_dest_path).and_return("tmp/template_test")
      end

      it 'should copy files from the template' do
        Appforce::Spawn::Template.copy_template(*[nil, {:client_api_name => 'template_test'}])
        expect(File.exist?("tmp/template_test/site.yml")).to be_truthy
      end

      after do
        `rm -rf tmp/template_test`
      end
    end
  end
  
  describe '.download_files' do
    context 'missing parameters (Client API Name)' do
      it 'should raise an exception' do
        expect { Appforce::Spawn::Template.download_files(*[nil, {}]) }.to raise_exception(Appforce::Spawn::MissingParameters)
      end
    end

    context 'it has correct parameters' do
      before do
        allow(Appforce::Spawn::Template).to receive(:ansible_dest_path).and_return("tmp/template_test")
        Appforce::Spawn::Template.copy_template(*[nil, {:client_api_name => 'template_test'}])
        allow(Appforce::Spawn::Api::Call).to receive(:get_hosts).and_return(file_fixture('hosts'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_active_users).and_return(file_fixture('users.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_inactive_users).and_return(file_fixture('inactive_users.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_vars).and_return(file_fixture('vars.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_main_scout).and_return(file_fixture('scout_main.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_all_host_data).and_return(file_fixture_to_json('all_host_data.json'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_host_scout_vars).and_return(file_fixture('host_scout_vars.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_private_key).and_return(file_fixture('fake_private_key.yml'))
        Appforce::Spawn::Template.create_dirs(*[nil, {:client_api_name => 'template_test'}])
      end

      it 'should copy files from the template' do
        Appforce::Spawn::Template.download_files(*[nil, {:client_api_name => 'template_test'}])
        expect(File.exist?("tmp/template_test/hosts")).to be_truthy
        expect(File.exist?("tmp/template_test/active_users.yml")).to be_truthy
        expect(File.exist?("tmp/template_test/inactive_users.yml")).to be_truthy
        expect(File.exist?("tmp/template_test/vars.yml")).to be_truthy
        expect(File.exist?("tmp/template_test/roles/scout/vars/ubuntu-test01.yml")).to be_truthy
        expect(File.exist?("tmp/template_test/roles/scout/vars/main.yml")).to be_truthy
      end

      after do
        `rm -rf tmp/template_test`
      end
    end

    context 'it has correct parameters, but there is an error writing files' do
      before do
        allow(Appforce::Spawn::Template).to receive(:ansible_dest_path).and_return("tmp/template_test")
        Appforce::Spawn::Template.copy_template(*[nil, {:client_api_name => 'template_test'}])
        allow(Appforce::Spawn::Api::Call).to receive(:get_hosts).and_return(file_fixture('hosts'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_active_users).and_return(file_fixture('users.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_inactive_users).and_return(file_fixture('inactive_users.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_vars).and_return(file_fixture('vars.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_main_scout).and_return(file_fixture('scout_main.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_all_host_data).and_return(file_fixture_to_json('all_host_data.json'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_host_scout_vars).and_return(file_fixture('host_scout_vars.yml'))
        allow(Appforce::Spawn::Api::Call).to receive(:get_private_key).and_return(file_fixture('fake_private_key.yml'))
        allow(File).to receive(:open).and_raise(IOError)
        Appforce::Spawn::Template.create_dirs(*[nil, {:client_api_name => 'template_test'}])
      end

      it 'should copy files from the template' do
        expect { Appforce::Spawn::Template.download_files(*[nil, {:client_api_name => 'template_test'}]) }.to raise_exception
      end

      after do
        `rm -rf tmp/template_test`
      end
    end
  end

end
